@@ -67,52 +67,52 @@ int localserver(const struct cfgstruct *copt)
 
     if((sockfd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
 	estr = strerror(errno);
-	logg("!Socket allocation error: %s\n", estr);
+	logg("!LOCAL: Socket allocation error: %s\n", estr);
 	return -1;
     }
 
     if(bind(sockfd, (struct sockaddr *) &server, sizeof(struct sockaddr_un)) == -1) {
 	if(errno == EADDRINUSE) {
 	    if(connect(sockfd, (struct sockaddr *) &server, sizeof(struct sockaddr_un)) >= 0) {
-		logg("!Socket file %s is in use by another process.\n", server.sun_path);
+		logg("!LOCAL: Socket file %s is in use by another process.\n", server.sun_path);
 		close(sockfd);
 		return -1;
 	    }
 	    if(cfgopt(copt, "FixStaleSocket")->enabled) {
-		logg("^Socket file %s exists. Unclean shutdown? Removing...\n", server.sun_path);
+		logg("#LOCAL: Removing stale socket file %s\n", server.sun_path);
 		if(unlink(server.sun_path) == -1) {
 		    estr = strerror(errno);
-		    logg("!Socket file %s could not be removed: %s\n", server.sun_path, estr);
+		    logg("!LOCAL: Socket file %s could not be removed: %s\n", server.sun_path, estr);
 		    close(sockfd);
 		    return -1;
 		}
 		if(bind(sockfd, (struct sockaddr *) &server, sizeof(struct sockaddr_un)) == -1) {
 		    estr = strerror(errno);
-		    logg("!Socket file %s could not be bound: %s (unlink tried)\n", server.sun_path, estr);
+		    logg("!LOCAL: Socket file %s could not be bound: %s (unlink tried)\n", server.sun_path, estr);
 		    close(sockfd);
 		    return -1;
 		}
 	    } else if(stat(server.sun_path, &foo) != -1) {
-		logg("!Socket file %s exists. Either remove it, or configure a different one.\n", server.sun_path);
+		logg("!LOCAL: Socket file %s exists. Either remove it, or configure a different one.\n", server.sun_path);
 		close(sockfd);
 		return -1;
 	    }
 	} else {
 	    estr = strerror(errno);
-	    logg("!Socket file %s could not be bound: %s\n", server.sun_path, estr);
+	    logg("!LOCAL: Socket file %s could not be bound: %s\n", server.sun_path, estr);
 	    close(sockfd);
 	    return -1;
 	}
     }
 
-    logg("Unix socket file %s\n", server.sun_path);
+    logg("#LOCAL: Unix socket file %s\n", server.sun_path);
 
     backlog = cfgopt(copt, "MaxConnectionQueueLength")->numarg;
-    logg("Setting connection queue length to %d\n", backlog);
+    logg("#LOCAL: Setting connection queue length to %d\n", backlog);
 
     if(listen(sockfd, backlog) == -1) {
 	estr = strerror(errno);
-	logg("!listen() error: %s\n", estr);
+	logg("!LOCAL: listen() error: %s\n", estr);
 	close(sockfd);
 	return -1;
     }
