@@ -49,7 +49,9 @@
 #include <time.h>
 #include <fcntl.h>
 #include <sys/stat.h>
+#include <dirent.h>
 #include <errno.h>
+#include <zlib.h>
 
 #include "manager.h"
 #include "notify.h"
@@ -63,12 +65,12 @@
 #include "shared/output.h"
 #include "shared/misc.h"
 #include "shared/cdiff.h"
+#include "shared/tar.h"
 
 #include "libclamav/clamav.h"
 #include "libclamav/others.h"
 #include "libclamav/str.h"
 #include "libclamav/cvd.h"
-#include "libclamav/lockdb.h"
 
 #ifndef	O_BINARY
 #define	O_BINARY	0
@@ -143,7 +145,7 @@ static int getclientsock(const char *localip)
     return socketfd;
 }
 
-static int wwwconnect(const char *server, const char *proxy, int pport, char *ip, const char *localip, int ctimeout, struct mirdat *mdat)
+static int wwwconnect(const char *server, const char *proxy, int pport, char *ip, const char *localip, int ctimeout, struct mirdat *mdat, int logerr)
 {
 	int socketfd = -1, port, i, ret;
 	struct sockaddr_in name;
@@ -209,7 +211,7 @@ static int wwwconnect(const char *server, const char *proxy, int pport, char *ip
 		herr = "Unknown error";
 		break;
 	}
-        logg("!Can't get information about %s: %s\n", hostpt, herr);
+        logg("%cCan't get information about %s: %s\n", logerr ? '!' : '^', hostpt, herr);
 	close(socketfd);
 	return -1;
     }
@@ -333,7 +335,7 @@ static char *proxyauth(const char *user, const char *pass)
     return auth;
 }
 
-static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int *ims, int ctimeout, int rtimeout, struct mirdat *mdat)
+static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int *ims, int ctimeout, int rtimeout, struct mirdat *mdat, int logerr)
 {
 	char cmd[512], head[513], buffer[FILEBUFF], ipaddr[16], *ch, *tmp;
 	int bread, cnt, sd;
@@ -397,9 +399,9 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
     memset(ipaddr, 0, sizeof(ipaddr));
 
     if(ip[0]) /* use ip to connect */
-	sd = wwwconnect(ip, proxy, port, ipaddr, localip, ctimeout, mdat);
+	sd = wwwconnect(ip, proxy, port, ipaddr, localip, ctimeout, mdat, logerr);
     else
-	sd = wwwconnect(hostname, proxy, port, ipaddr, localip, ctimeout, mdat);
+	sd = wwwconnect(hostname, proxy, port, ipaddr, localip, ctimeout, mdat, logerr);
 
     if(sd < 0) {
 	return NULL;
@@ -412,7 +414,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 	strcpy(ip, ipaddr);
 
     if(send(sd, cmd, strlen(cmd), 0) < 0) {
-	logg("!remote_cvdhead: write failed\n");
+	logg("%cremote_cvdhead: write failed\n", logerr ? '!' : '^');
 	closesocket(sd);
 	return NULL;
     }
@@ -432,13 +434,13 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
     closesocket(sd);
 
     if(bread == -1) {
-	logg("!remote_cvdhead: Error while reading CVD header from %s\n", hostname);
+	logg("%cremote_cvdhead: Error while reading CVD header from %s\n", logerr ? '!' : '^', hostname);
 	mirman_update(mdat->currip, mdat, 1);
 	return NULL;
     }
 
     if((strstr(buffer, "HTTP/1.1 404")) != NULL || (strstr(buffer, "HTTP/1.0 404")) != NULL) { 
-	logg("!CVD file not found on remote server\n");
+	logg("%cCVD file not found on remote server\n", logerr ? '!' : '^');
 	/* mirman_update(mdat->currip, mdat, 1); */
 	return NULL;
     }
@@ -455,7 +457,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 
     if(!strstr(buffer, "HTTP/1.1 200") && !strstr(buffer, "HTTP/1.0 200") &&
        !strstr(buffer, "HTTP/1.1 206") && !strstr(buffer, "HTTP/1.0 206")) {
-	logg("!Unknown response from remote server\n");
+	logg("%cUnknown response from remote server\n", logerr ? '!' : '^');
 	mirman_update(mdat->currip, mdat, 1);
 	return NULL;
     }
@@ -473,7 +475,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
     }
 
     if(sizeof(buffer) - i < 512) {
-	logg("!remote_cvdhead: Malformed CVD header (too short)\n");
+	logg("%cremote_cvdhead: Malformed CVD header (too short)\n", logerr ? '!' : '^');
 	mirman_update(mdat->currip, mdat, 1);
 	return NULL;
     }
@@ -482,7 +484,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
 
     for(j = 0; j < 512; j++) {
 	if(!ch || (ch && !*ch) || (ch && !isprint(ch[j]))) {
-	    logg("!remote_cvdhead: Malformed CVD header (bad chars)\n");
+	    logg("%cremote_cvdhead: Malformed CVD header (bad chars)\n", logerr ? '!' : '^');
 	    mirman_update(mdat->currip, mdat, 1);
 	    return NULL;
 	}
@@ -490,7 +492,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
     }
 
     if(!(cvd = cl_cvdparse(head))) {
-	logg("!Malformed CVD header (can't parse)\n");
+	logg("%cremote_cvdhead: Malformed CVD header (can't parse)\n", logerr ? '!' : '^');
 	mirman_update(mdat->currip, mdat, 1);
     } else {
 	logg("OK\n");
@@ -500,7 +502,7 @@ static struct cl_cvd *remote_cvdhead(const char *file, const char *hostname, cha
     return cvd;
 }
 
-static int getfile(const char *srcfile, const char *destfile, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int ctimeout, int rtimeout, struct mirdat *mdat)
+static int getfile(const char *srcfile, const char *destfile, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int ctimeout, int rtimeout, struct mirdat *mdat, int logerr)
 {
 	char cmd[512], buffer[FILEBUFF], *ch;
 	int bread, fd, totalsize = 0,  rot = 0, totaldownloaded = 0,
@@ -547,9 +549,9 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
     memset(ipaddr, 0, sizeof(ipaddr));
 
     if(ip[0]) /* use ip to connect */
-	sd = wwwconnect(ip, proxy, port, ipaddr, localip, ctimeout, mdat);
+	sd = wwwconnect(ip, proxy, port, ipaddr, localip, ctimeout, mdat, logerr);
     else
-	sd = wwwconnect(hostname, proxy, port, ipaddr, localip, ctimeout, mdat);
+	sd = wwwconnect(hostname, proxy, port, ipaddr, localip, ctimeout, mdat, logerr);
 
     if(sd < 0) {
 	return 52;
@@ -561,7 +563,7 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
 	strcpy(ip, ipaddr);
 
     if(send(sd, cmd, strlen(cmd), 0) < 0) {
-	logg("!getfile: Can't write to socket\n");
+	logg("%cgetfile: Can't write to socket\n", logerr ? '!' : '^');
 	return 52;
     }
 
@@ -581,7 +583,7 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
 #else
 	if((i >= sizeof(buffer) - 1) || recv(sd, buffer + i, 1, 0) == -1) {
 #endif
-	    logg("!getfile: Error while reading database from %s (IP: %s)\n", hostname, ipaddr);
+	    logg("%cgetfile: Error while reading database from %s (IP: %s)\n", logerr ? '!' : '^', hostname, ipaddr);
 	    mirman_update(mdat->currip, mdat, 1);
 	    return 52;
 	}
@@ -606,7 +608,7 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
 
     if(!strstr(buffer, "HTTP/1.1 200") && !strstr(buffer, "HTTP/1.0 200") &&
        !strstr(buffer, "HTTP/1.1 206") && !strstr(buffer, "HTTP/1.0 206")) {
-	logg("!getfile: Unknown response from remote server (IP: %s)\n", ipaddr);
+	logg("%cgetfile: Unknown response from remote server (IP: %s)\n", logerr ? '!' : '^', ipaddr);
 	mirman_update(mdat->currip, mdat, 1);
 	closesocket(sd);
 	return 58;
@@ -676,97 +678,78 @@ static int getfile(const char *srcfile, const char *destfile, const char *hostna
     return 0;
 }
 
-static int getcvd(const char *dbfile, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int nodb, unsigned int newver, int ctimeout, int rtimeout, struct mirdat *mdat)
+static int getcvd(const char *cvdfile, const char *newfile, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int nodb, unsigned int newver, int ctimeout, int rtimeout, struct mirdat *mdat, int logerr)
 {
-	char *tempname;
 	struct cl_cvd *cvd;
 	int ret;
 
 
-    tempname = cli_gentemp(".");
-
-    logg("*Retrieving http://%s/%s\n", hostname, dbfile);
-    if((ret = getfile(dbfile, tempname, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat))) {
-        logg("!Can't download %s from %s\n", dbfile, hostname);
-        unlink(tempname);
-        free(tempname);
+    logg("*Retrieving http://%s/%s\n", hostname, cvdfile);
+    if((ret = getfile(cvdfile, newfile, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat, logerr))) {
+        logg("%cCan't download %s from %s\n", logerr ? '!' : '^', cvdfile, hostname);
+        unlink(newfile);
         return ret;
     }
 
-    if((ret = cl_cvdverify(tempname))) {
+    if((ret = cl_cvdverify(newfile))) {
         logg("!Verification: %s\n", cl_strerror(ret));
-        unlink(tempname);
-        free(tempname);
+        unlink(newfile);
         return 54;
     }
 
-    if(!(cvd = cl_cvdhead(tempname))) {
-	logg("!Can't read CVD header of new %s database.\n", dbfile);
-	unlink(tempname);
-	free(tempname);
+    if(!(cvd = cl_cvdhead(newfile))) {
+	logg("!Can't read CVD header of new %s database.\n", cvdfile);
+	unlink(newfile);
 	return 54;
     }
 
     if(cvd->version < newver) {
 	logg("^Mirror %s is not synchronized.\n", ip);
     	cl_cvdfree(cvd);
-	unlink(tempname);
-	free(tempname);
+	unlink(newfile);
 	return 59;
     }
 
-    if(!nodb && unlink(dbfile)) {
-	logg("!Can't unlink %s. Please fix it and try again.\n", dbfile);
-    	cl_cvdfree(cvd);
-	unlink(tempname);
-	free(tempname);
-	return 53;
-    } else {
-    	if(rename(tempname, dbfile) == -1) {
-    	    if(errno == EEXIST) {
-    	        unlink(dbfile);
-    	        if(rename(tempname, dbfile) == -1) {
-                   logg("!All attempts to rename the temporary file failed: %s\n", strerror(errno));
-		   return 57;
-		}
-            }
-        }
-    }
-
+    cl_cvdfree(cvd);
     return 0;
 }
 
-static int chdir_inc(const char *dbname)
+static int chdir_tmp(const char *dbname, const char *tmpdir)
 {
-	struct stat sb;
-	char path[32], dbfile[32];
+	char cvdfile[32];
 
 
-    sprintf(path, "%s.inc", dbname);
-    sprintf(dbfile, "%s.cvd", dbname);
+    if(access(tmpdir, R_OK|W_OK) == -1) {
+	sprintf(cvdfile, "%s.cvd", dbname);
+	if(access(cvdfile, R_OK) == -1) {
+	    sprintf(cvdfile, "%s.cld", dbname);
+	    if(access(cvdfile, R_OK) == -1) {
+		logg("!chdir_tmp: Can't access local %s database\n", dbname);
+		return -1;
+	    }
+	}
 
-    if(stat(path, &sb) == -1) {
-	if(mkdir(path, 0755) == -1) {
-	    logg("!chdir_inc: Can't create directory %s\n", path);
+	if(mkdir(tmpdir, 0755) == -1) {
+	    logg("!chdir_tmp: Can't create directory %s\n", tmpdir);
 	    return -1;
 	}
 
-	if(cvd_unpack(dbfile, path) == -1) {
-	    logg("!chdir_inc: Can't unpack %s into %s\n", dbfile, path);
-	    cli_rmdirs(path);
+	if(cvd_unpack(cvdfile, tmpdir) == -1) {
+	    logg("!chdir_tmp: Can't unpack %s into %s\n", cvdfile, tmpdir);
+	    cli_rmdirs(tmpdir);
 	    return -1;
 	}
     }
 
-    if(chdir(path) == -1) {
-	logg("!chdir_inc: Can't change directory to %s\n", path);
+    if(chdir(tmpdir) == -1) {
+	logg("!chdir_tmp: Can't change directory to %s\n", tmpdir);
 	return -1;
     }
 
     return 0;
 }
 
-static int getpatch(const char *dbname, int version, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int ctimeout, int rtimeout, struct mirdat *mdat)
+static int getpatch(const char *dbname, const char *tmpdir, int version, const char *hostname, char *ip, const char *localip, const char *proxy, int port, const char *user, const char *pass, const char *uas, int ctimeout, int rtimeout, struct mirdat *mdat, int logerr)
 {
 	char *tempname, patch[32], olddir[512];
 	int ret, fd;
@@ -777,15 +760,15 @@ static int getpatch(const char *dbname, int version, const char *hostname, char
 	return 50; /* FIXME */
     }
 
-    if(chdir_inc(dbname) == -1)
+    if(chdir_tmp(dbname, tmpdir) == -1)
 	return 50;
 
     tempname = cli_gentemp(".");
     snprintf(patch, sizeof(patch), "%s-%d.cdiff", dbname, version);
 
     logg("*Retrieving http://%s/%s\n", hostname, patch);
-    if((ret = getfile(patch, tempname, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat))) {
-        logg("!getpatch: Can't download %s from %s\n", patch, hostname);
+    if((ret = getfile(patch, tempname, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat, logerr))) {
+        logg("%cgetpatch: Can't download %s from %s\n", logerr ? '!' : '^', patch, hostname);
         unlink(tempname);
         free(tempname);
 	chdir(olddir);
@@ -816,64 +799,185 @@ static int getpatch(const char *dbname, int version, const char *hostname, char
     return 0;
 }
 
-static struct cl_cvd *currentdb(const char *dbname, unsigned int *inc)
+static struct cl_cvd *currentdb(const char *dbname, char *localname)
 {
-	struct stat sb;
-	char path[512];
+	char db[32];
 	struct cl_cvd *cvd = NULL;
 
 
-    snprintf(path, sizeof(path), "%s.inc", dbname);
-    if(stat(path, &sb) != -1) {
-	snprintf(path, sizeof(path), "%s.inc/%s.info", dbname, dbname);
-	if(inc)
-	    *inc = 1;
-    } else {
-	snprintf(path, sizeof(path), "%s.cvd", dbname);
-	if(inc)
-	    *inc = 0;
+    snprintf(db, sizeof(db), "%s.cvd", dbname);
+    if(localname)
+	strcpy(localname, db);
+
+    if(access(db, R_OK) == -1) {
+	snprintf(db, sizeof(db), "%s.cld", dbname);
+	if(localname)
+	    strcpy(localname, db);
     }
 
-    if(!access(path, R_OK))
-	cvd = cl_cvdhead(path);
+    if(!access(db, R_OK))
+	cvd = cl_cvdhead(db);
 
     return cvd;
 }
 
-static int updatedb(const char *dbname, const char *hostname, char *ip, int *signo, const struct cfgstruct *copt, const char *dnsreply, char *localip, int outdated, struct mirdat *mdat)
+static int buildcld(const char *tmpdir, const char *dbname, const char *newfile, unsigned int compr)
+{
+	DIR *dir;
+	char cwd[512], info[32], buff[512], *pt;
+	struct dirent *dent;
+	int fd, err = 0;
+	gzFile *gzs = NULL;
+
+
+    getcwd(cwd, sizeof(cwd));
+    if(chdir(tmpdir) == -1) {
+	logg("!buildcld: Can't access directory %s\n", tmpdir);
+	return -1;
+    }
+
+    snprintf(info, sizeof(info), "%s.info", dbname);
+    if((fd = open(info, O_RDONLY|O_BINARY)) == -1) {
+	logg("!buildcld: Can't open %s\n", info);
+	chdir(cwd);
+	return -1;
+    }
+
+    if(read(fd, buff, 512) == -1) {
+	logg("!buildcld: Can't read %s\n", info);
+	chdir(cwd);
+	close(fd);
+	return -1;
+    }
+    close(fd);
+
+    if(!(pt = strchr(buff, '\n'))) {
+	logg("!buildcld: Bad format of %s\n", info);
+	chdir(cwd);
+	return -1;
+    }
+    memset(pt, ' ', 512 + buff - pt);
+
+    if((fd = open(newfile, O_WRONLY|O_CREAT|O_EXCL|O_BINARY, 0644)) == -1) {
+	logg("!buildcld: Can't open %s for writing\n", newfile);
+	chdir(cwd);
+	return -1;
+    }
+    if(write(fd, buff, 512) != 512) {
+	logg("!buildcld: Can't write to %s\n", newfile);
+	chdir(cwd);
+	unlink(newfile);
+	close(fd);
+	return -1;
+    }
+
+    if((dir = opendir(".")) == NULL) {
+	logg("!buildcld: Can't open directory %s\n", tmpdir);
+	chdir(cwd);
+	unlink(newfile);
+	close(fd);
+	return -1;
+    }
+
+    if(compr) {
+	close(fd);
+	if(!(gzs = gzopen(newfile, "ab"))) {
+	    logg("!buildcld: gzopen() failed for %s\n", newfile);
+	    chdir(cwd);
+	    unlink(newfile);
+	    closedir(dir);
+	    return -1;
+	}
+    }
+
+    if(access("COPYING", R_OK)) {
+	logg("!buildcld: COPYING file not found\n");
+	err = 1;
+    } else {
+	if(tar_addfile(fd, gzs, "COPYING") == -1) {
+	    logg("!buildcld: Can't add COPYING to .cld file\n");
+	    err = 1;
+	}
+    }
+
+    if(!err && !access("daily.cfg", R_OK)) {
+	if(tar_addfile(fd, gzs, "daily.cfg") == -1) {
+	    logg("!buildcld: Can't add daily.cfg to .cld file\n");
+	    err = 1;
+	}
+    }
+
+    if(err) {
+	chdir(cwd);
+	if(gzs)
+	    gzclose(gzs);
+	else
+	    close(fd);
+	unlink(newfile);
+	return -1;
+    }
+
+    while((dent = readdir(dir))) {
+#ifndef C_INTERIX
+	if(dent->d_ino)
+#endif
+	{
+	    if(!strcmp(dent->d_name, ".") || !strcmp(dent->d_name, "..") || !strcmp(dent->d_name, "COPYING") || !strcmp(dent->d_name, "daily.cfg"))
+		continue;
+
+	    if(tar_addfile(fd, gzs, dent->d_name) == -1) {
+		logg("!buildcld: Can't add %s to .cld file\n", dent->d_name);
+		chdir(cwd);
+		if(gzs)
+		    gzclose(gzs);
+		else
+		    close(fd);
+		unlink(newfile);
+		closedir(dir);
+		return -1;
+	    }
+	}
+    }
+    closedir(dir);
+
+    if(gzs) {
+	if(gzclose(gzs)) {
+	    logg("!buildcld: gzclose() failed for %s\n", newfile);
+	    unlink(newfile);
+	    return -1;
+	}
+    } else {
+	if(close(fd) == -1) {
+	    logg("!buildcld: close() failed for %s\n", newfile);
+	    unlink(newfile);
+	    return -1;
+	}
+    }
+
+    if(chdir(cwd) == -1) {
+	logg("!buildcld: Can't return to previous directory %s\n", cwd);
+	return -1;
+    }
+
+    return 0;
+}
+
+static int updatedb(const char *dbname, const char *hostname, char *ip, int *signo, const struct cfgstruct *copt, const char *dnsreply, char *localip, int outdated, struct mirdat *mdat, int logerr)
 {
 	struct cl_cvd *current, *remote;
 	const struct cfgstruct *cpt;
 	unsigned int nodb = 0, currver = 0, newver = 0, port = 0, i, j;
 	int ret, ims = -1;
-	char *pt, dbfile[32], dbinc[32], *bacinc = NULL;
+	char *pt, cvdfile[32], localname[32], *tmpdir = NULL, *newfile, newdb[32], cwd[512];
 	const char *proxy = NULL, *user = NULL, *pass = NULL, *uas = NULL;
-	unsigned int flevel = cl_retflevel(), inc, maxattempts;
-	struct stat sb;
+	unsigned int flevel = cl_retflevel(), maxattempts;
 	int ctimeout, rtimeout;
 
 
-    snprintf(dbfile, sizeof(dbfile), "%s.cvd", dbname);
-    snprintf(dbinc, sizeof(dbinc), "%s.inc", dbname);
+    snprintf(cvdfile, sizeof(cvdfile), "%s.cvd", dbname);
 
-    if(!(current = currentdb(dbname, &inc))) {
+    if(!(current = currentdb(dbname, localname))) {
 	nodb = 1;
-	inc = 0;
-	if(stat(dbinc, &sb) != -1) {
-	    logg("^Removing corrupted incremental directory %s\n", dbinc);
-	    if(cli_rmdirs(dbinc)) {
-		logg("!Can't remove incremental directory\n");
-		return 53;
-	    }
-
-	    if(stat(dbfile, &sb) != -1) {
-		logg("^Removing obsolete %s\n", dbfile);
-		if(unlink(dbfile)) {
-		    logg("!Can't unlink %s\n", dbfile);
-		    return 53;
-		}
-	    }
-	}
     } else {
 	mdat->dbflevel = current->fl;
     }
@@ -892,11 +996,11 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	}
 
 	if(field && (pt = cli_strtok(dnsreply, field, ":"))) {
-	    if(!isnumb(pt)) {
+	    if(!cli_isnumber(pt)) {
 		logg("^Broken database version in TXT record.\n");
 	    } else {
 		newver = atoi(pt);
-		logg("*%s version from DNS: %d\n", dbfile, newver);
+		logg("*%s version from DNS: %d\n", cvdfile, newver);
 	    }
 	    free(pt);
 	} else {
@@ -936,17 +1040,17 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 
     if(!nodb && !newver) {
 
-	remote = remote_cvdhead(dbfile, hostname, ip, localip, proxy, port, user, pass, uas, &ims, ctimeout, rtimeout, mdat);
+	remote = remote_cvdhead(cvdfile, hostname, ip, localip, proxy, port, user, pass, uas, &ims, ctimeout, rtimeout, mdat, logerr);
 
 	if(!nodb && !ims) {
-	    logg("%s is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n", inc ? dbinc : dbfile, current->version, current->sigs, current->fl, current->builder);
+	    logg("%s is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n", localname, current->version, current->sigs, current->fl, current->builder);
 	    *signo += current->sigs;
 	    cl_cvdfree(current);
 	    return 1;
 	}
 
 	if(!remote) {
-	    logg("^Can't read %s header from %s (IP: %s)\n", dbfile, hostname, ip);
+	    logg("^Can't read %s header from %s (IP: %s)\n", cvdfile, hostname, ip);
 	    cl_cvdfree(current);
 	    return 58;
 	}
@@ -956,7 +1060,7 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
     }
 
     if(!nodb && (current->version >= newver)) {
-	logg("%s is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n", inc ? dbinc : dbfile, current->version, current->sigs, current->fl, current->builder);
+	logg("%s is up to date (version: %d, sigs: %d, f-level: %d, builder: %s)\n", localname, current->version, current->sigs, current->fl, current->builder);
 
 	if(!outdated && flevel < current->fl) {
 	    /* display warning even for already installed database */
@@ -997,29 +1101,29 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
     if(!cfgopt(copt, "ScriptedUpdates")->enabled)
 	nodb = 1;
 
+    getcwd(cwd, sizeof(cwd));
+    newfile = cli_gentemp(cwd);
+
     if(nodb) {
-	ret = getcvd(dbfile, hostname, ip, localip, proxy, port, user, pass, uas, nodb, newver, ctimeout, rtimeout, mdat);
+	ret = getcvd(cvdfile, newfile, hostname, ip, localip, proxy, port, user, pass, uas, nodb, newver, ctimeout, rtimeout, mdat, logerr);
 	if(ret) {
 	    memset(ip, 0, 16);
+	    free(newfile);
 	    return ret;
 	}
+	snprintf(newdb, sizeof(newdb), "%s.cvd", dbname);
 
     } else {
 	ret = 0;
 
-	if(!access(dbinc, X_OK)) {
-	    if((bacinc = cli_gentemp("."))) {
-		if(dircopy(dbinc, bacinc) == -1) {
-		    free(bacinc);
-		    bacinc = NULL;
-		}
-	    }
-	}
-
+	tmpdir = cli_gentemp(".");
 	maxattempts = cfgopt(copt, "MaxAttempts")->numarg;
 	for(i = currver + 1; i <= newver; i++) {
 	    for(j = 0; j < maxattempts; j++) {
-		ret = getpatch(dbname, i, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat);
+		    int llogerr = logerr;
+		if(logerr)
+		    llogerr = (j == maxattempts - 1);
+		ret = getpatch(dbname, tmpdir, i, hostname, ip, localip, proxy, port, user, pass, uas, ctimeout, rtimeout, mdat, llogerr);
 		if(ret == 52 || ret == 58) {
 		    memset(ip, 0, 16);
 		    continue;
@@ -1032,43 +1136,52 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
 	}
 
 	if(ret) {
-	    logg("^Incremental update failed, trying to download %s\n", dbfile);
-
-	    ret = getcvd(dbfile, hostname, ip, localip, proxy, port, user, pass, uas, 1, newver, ctimeout, rtimeout, mdat);
+	    cli_rmdirs(tmpdir);
+	    free(tmpdir);
+	    logg("^Incremental update failed, trying to download %s\n", cvdfile);
+	    ret = getcvd(cvdfile, newfile, hostname, ip, localip, proxy, port, user, pass, uas, 1, newver, ctimeout, rtimeout, mdat, logerr);
 	    if(ret) {
-		if(bacinc) {
-		    logg("*Restoring incremental directory %s from backup\n", dbinc);
-		    cli_rmdirs(dbinc);
-		    rename(bacinc, dbinc);
-		    free(bacinc);
-		}
+		free(newfile);
 		return ret;
-	    } else {
-		logg("*Removing incremental directory %s\n", dbinc);
-		cli_rmdirs(dbinc);
 	    }
-
+	    snprintf(newdb, sizeof(newdb), "%s.cvd", dbname);
 	} else {
-	    unlink(dbfile);
-	}
-
-	if(bacinc) {
-	    logg("*Removing backup directory %s\n", bacinc);
-	    cli_rmdirs(bacinc);
-	    free(bacinc);
+	    if(buildcld(tmpdir, dbname, newfile, cfgopt(copt, "CompressLocalDatabase")->enabled) == -1) {
+		logg("!Can't create local database\n");
+		cli_rmdirs(tmpdir);
+		free(tmpdir);
+		free(newfile);
+		return 70; /* FIXME */
+	    }
+	    snprintf(newdb, sizeof(newdb), "%s.cld", dbname);
+	    cli_rmdirs(tmpdir);
+	    free(tmpdir);
 	}
     }
 
-    if(!(current = currentdb(dbname, NULL))) {
-	/* should never be reached */
-	logg("!Can't parse new database\n");
+    if(!(current = cl_cvdhead(newfile))) {
+	logg("!Can't parse new database %s\n", newfile);
+	unlink(newfile);
+	free(newfile);
 	return 55; /* FIXME */
     }
 
-    if(nodb && inc)
-	cli_rmdirs(dbinc);
+    if(!nodb && !access(localname, R_OK) && unlink(localname)) {
+	logg("!Can't unlink %s. Please fix it and try again.\n", localname);
+	unlink(newfile);
+	free(newfile);
+	return 53;
+    }
 
-    logg("%s updated (version: %d, sigs: %d, f-level: %d, builder: %s)\n", inc ? dbinc : dbfile, current->version, current->sigs, current->fl, current->builder);
+    if(rename(newfile, newdb) == -1) {
+	logg("!Can't rename %s to %s\n", newfile, newdb);
+	unlink(newfile);
+	free(newfile);
+	return 57;
+    }
+    free(newfile);
+
+    logg("%s updated (version: %d, sigs: %d, f-level: %d, builder: %s)\n", newdb, current->version, current->sigs, current->fl, current->builder);
 
     if(flevel < current->fl) {
 	logg("^Your ClamAV installation is OUTDATED!\n");
@@ -1081,11 +1194,11 @@ static int updatedb(const char *dbname, const char *hostname, char *ip, int *sig
     return 0;
 }
 
-int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, const char *hostname, const char *dbdir)
+int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, const char *hostname, const char *dbdir, int logerr)
 {
 	time_t currtime;
 	int ret, updated = 0, outdated = 0, signo = 0;
-	unsigned int ttl, try = 0;
+	unsigned int ttl;
 	char ipaddr[16], *dnsreply = NULL, *pt, *localip = NULL, *newver = NULL;
 	const char *arg = NULL;
 	const struct cfgstruct *cpt;
@@ -1097,7 +1210,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
     time(&currtime);
     logg("ClamAV update process started at %s", ctime(&currtime));
 
-#ifndef HAVE_GMP
+#ifndef HAVE_LIBGMP
     logg("SECURITY WARNING: NO SUPPORT FOR DIGITAL SIGNATURES\n");
     logg("See the FAQ at http://www.clamav.net/support/faq for an explanation.\n");
 #endif
@@ -1173,19 +1286,6 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	localip = cpt->strarg;
     }
 
-    while(cli_writelockdb(dbdir, 0) == CL_ELOCKDB) {
-	logg("*Waiting to lock database directory: %s\n", dbdir);
-	sleep(5);
-	if(++try > 12) {
-	    logg("!Can't lock database directory: %s\n", dbdir);
-	    if(dnsreply)
-		free(dnsreply);
-	    if(newver)
-		free(newver);
-	    return 61; 
-	}
-    }
-
     if(cfgopt(copt, "HTTPProxyServer")->enabled)
 	mirman_read("mirrors.dat", &mdat, 0);
     else
@@ -1193,7 +1293,7 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 
     memset(ipaddr, 0, sizeof(ipaddr));
 
-    if((ret = updatedb("main", hostname, ipaddr, &signo, copt, dnsreply, localip, outdated, &mdat)) > 50) {
+    if((ret = updatedb("main", hostname, ipaddr, &signo, copt, dnsreply, localip, outdated, &mdat, logerr)) > 50) {
 	if(dnsreply)
 	    free(dnsreply);
 
@@ -1201,14 +1301,13 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
-	cli_unlockdb(dbdir);
 	return ret;
 
     } else if(ret == 0)
 	updated = 1;
 
     /* if ipaddr[0] != 0 it will use it to connect to the web host */
-    if((ret = updatedb("daily", hostname, ipaddr, &signo, copt, dnsreply, localip, outdated, &mdat)) > 50) {
+    if((ret = updatedb("daily", hostname, ipaddr, &signo, copt, dnsreply, localip, outdated, &mdat, logerr)) > 50) {
 	if(dnsreply)
 	    free(dnsreply);
 
@@ -1216,7 +1315,6 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	    free(newver);
 
 	mirman_write("mirrors.dat", &mdat);
-	cli_unlockdb(dbdir);
 	return ret;
 
     } else if(ret == 0)
@@ -1226,7 +1324,6 @@ int downloadmanager(const struct cfgstruct *copt, const struct optstruct *opt, c
 	free(dnsreply);
 
     mirman_write("mirrors.dat", &mdat);
-    cli_unlockdb(dbdir);
 
     if(updated) {
 	if(cfgopt(copt, "HTTPProxyServer")->enabled) {
