@@ -1,14 +1,13 @@
 /*
  *  Extract component parts of OLE2 files (e.g. MS Office Documents)
  *
- *  Copyright (C) 2004 trog@uncon.org
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Trog
  *
- *  This code is based on the OpenOffice and libgsf sources.
- *                  
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,8 +23,8 @@
 #ifndef __OLE2_EXTRACT_H
 #define __OLE2_EXTRACT_H
 
-#include "clamav.h"
+#include "others.h"
 
-int cli_ole2_extract(int fd, const char *dirname, const struct cl_limits *limits);
+int cli_ole2_extract(int fd, const char *dirname, cli_ctx *ctx);
 
 #endif
