@@ -1,10 +1,11 @@
 /*
- *  Copyright (C) 2005 Nigel Horne <njh@bandsman.co.uk>
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Nigel Horne
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -16,10 +17,11 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  *  MA 02110-1301, USA.
  */
-
 #ifndef __PDF_H
 #define __PDF_H
 
-int cli_pdf(const char *dir, int desc, const cli_ctx *ctx);
+#include "others.h"
+
+int cli_pdf(const char *dir, int desc, cli_ctx *ctx);
 
 #endif
