@@ -1,14 +1,13 @@
 /*
  *  Extract VBA source code for component MS Office Documents
  *
- *  Copyright (C) 2004 trog@uncon.org
+ *  Copyright (C) 2007-2008 Sourcefire, Inc.
+ *
+ *  Authors: Trog, Nigel Horne
  *
- *  This code is based on the OpenOffice and libgsf sources.
- *                  
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU General Public License version 2 as
+ *  published by the Free Software Foundation.
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,22 +26,20 @@
 #include "cltypes.h"
 
 typedef struct vba_project_tag {
-	int count;
 	char **name;
 	uint32_t *offset;
 	uint32_t *length;	/* for Word 6 macros */
 	unsigned char *key;	/* for Word 6 macros */
 	char *dir;
+	int count;
 } vba_project_t;
 
-vba_project_t *vba56_dir_read(const char *dir);
-unsigned char *vba_decompress(int fd, uint32_t offset, int *size);
-int cli_decode_ole_object(int fd, const char *dir);
-
-char *ppt_vba_read(const char *dir);
-
-vba_project_t *wm_dir_read(const char *dir);
-unsigned char *wm_decrypt_macro(int fd, uint32_t offset, uint32_t len,
+vba_project_t	*cli_vba_readdir(const char *dir);
+unsigned char	*cli_vba_inflate(int fd, off_t offset, int *size);
+int	cli_decode_ole_object(int fd, const char *dir);
+char	*cli_ppt_vba_read(const char *filename);
+vba_project_t	*cli_wm_readdir(const char *dir);
+unsigned char	*cli_wm_decrypt_macro(int fd, off_t offset, uint32_t len,
 					unsigned char key);
 
 #endif
