@@ -47,7 +47,6 @@ struct cfgoption cfg_options[] = {
     {"DetectBrokenExecutables", OPT_BOOL, 0, NULL, 0, OPT_CLAMD},
     {"ScanMail", OPT_BOOL, 1, NULL, 0, OPT_CLAMD},
     {"MailFollowURLs", OPT_BOOL, 0, NULL, 0, OPT_CLAMD},
-    {"MailMaxRecursion", OPT_NUM, 64, NULL, 0, OPT_CLAMD},
     {"PhishingSignatures", OPT_BOOL, 1, NULL, 0, OPT_CLAMD},
     {"PhishingScanURLs",OPT_BOOL, 1, NULL, 0, OPT_CLAMD},
     /* these are FP prone options, if default isn't used */
@@ -61,13 +60,12 @@ struct cfgoption cfg_options[] = {
     {"ScanOLE2", OPT_BOOL, 1, NULL, 0, OPT_CLAMD},
     {"ScanPDF", OPT_BOOL, 0, NULL, 0, OPT_CLAMD},
     {"ScanArchive", OPT_BOOL, 1, NULL, 0, OPT_CLAMD},
-    {"ArchiveMaxFileSize", OPT_COMPSIZE, 10485760, NULL, 0, OPT_CLAMD},
-    {"ArchiveMaxRecursion", OPT_NUM, 8, NULL, 0, OPT_CLAMD},
-    {"ArchiveMaxFiles", OPT_NUM, 1000, NULL, 0, OPT_CLAMD},
-    {"ArchiveMaxCompressionRatio", OPT_NUM, 250, NULL, 0, OPT_CLAMD},
+    {"MaxScanSize", OPT_COMPSIZE, 104857600, NULL, 0, OPT_CLAMD},
+    {"MaxFileSize", OPT_COMPSIZE, 26214400, NULL, 0, OPT_CLAMD},
+    {"MaxRecursion", OPT_NUM, 16, NULL, 0, OPT_CLAMD},
+    {"MaxFiles", OPT_NUM, 10000, NULL, 0, OPT_CLAMD},
     {"ArchiveLimitMemoryUsage", OPT_BOOL, 0, NULL, 0, OPT_CLAMD},
     {"ArchiveBlockEncrypted", OPT_BOOL, 0, NULL, 0, OPT_CLAMD},
-    {"ArchiveBlockMax", OPT_BOOL, 0, NULL, 0, OPT_CLAMD},
     {"DatabaseDirectory", OPT_QUOTESTR, -1, DATADIR, 0, OPT_CLAMD | OPT_FRESHCLAM},
     {"TCPAddr", OPT_QUOTESTR, -1, NULL, 0, OPT_CLAMD},
     {"TCPSocket", OPT_NUM, -1, NULL, 0, OPT_CLAMD},
@@ -105,6 +103,7 @@ struct cfgoption cfg_options[] = {
     {"DatabaseMirror", OPT_QUOTESTR, -1, NULL, 1, OPT_FRESHCLAM},
     {"MaxAttempts", OPT_NUM, 3, NULL, 0, OPT_FRESHCLAM},
     {"ScriptedUpdates", OPT_BOOL, 1, NULL, 0, OPT_FRESHCLAM},
+    {"CompressLocalDatabase", OPT_BOOL, 0, NULL, 0, OPT_FRESHCLAM},
     {"HTTPProxyServer", OPT_QUOTESTR, -1, NULL, 0, OPT_FRESHCLAM},
     {"HTTPProxyPort", OPT_NUM, -1, NULL, 0, OPT_FRESHCLAM},
     {"HTTPProxyUsername", OPT_QUOTESTR, -1, NULL, 0, OPT_FRESHCLAM},
@@ -257,7 +256,7 @@ struct cfgstruct *getcfg(const char *cfgfile, int verbose)
 				}
 				break;
 			    case OPT_NUM:
-				if(!arg || !isnumb(arg)) {
+				if(!arg || !cli_isnumber(arg)) {
 				    if(verbose)
 					fprintf(stderr, "ERROR: Parse error at line %d: Option %s requires numerical argument.\n", line, name);
 				    fclose(fs);
@@ -289,7 +288,7 @@ struct cfgstruct *getcfg(const char *cfgfile, int verbose)
 				if(ctype == 'm' || ctype == 'k') {
 				    char *cpy = (char *) calloc(strlen(arg), 1);
 				    strncpy(cpy, arg, strlen(arg) - 1);
-				    if(!isnumb(cpy)) {
+				    if(!cli_isnumber(cpy)) {
 					if(verbose)
 					    fprintf(stderr, "ERROR: Parse error at line %d: Option %s requires numerical (raw/K/M) argument.\n", line, name);
 					fclose(fs);
@@ -304,7 +303,7 @@ struct cfgstruct *getcfg(const char *cfgfile, int verbose)
 					calc = atoi(cpy) * 1024;
 				    free(cpy);
 				} else {
-				    if(!isnumb(arg)) {
+				    if(!cli_isnumber(arg)) {
 					if(verbose)
 					    fprintf(stderr, "ERROR: Parse error at line %d: Option %s requires numerical (raw/K/M) argument.\n", line, name);
 					fclose(fs);
